import React from "react";
import "./historynavigator.css";
import { Checkpoint, History } from "../../common/history";

interface HistoryNavigatorProps<HistoryDataType> {
    history: History<HistoryDataType>;
    onCheckpointLoad: { (checkpoint: Checkpoint<HistoryDataType>): void };
}

function HistoryNavigator<HistoryDataType>({
    history,
    onCheckpointLoad,
}: HistoryNavigatorProps<HistoryDataType>) {
    return (
        <div id="history-navigator">
            <button onClick={() => onCheckpointLoad(history.undo())}>
                Undo
            </button>
            <button onClick={() => onCheckpointLoad(history.redo())}>
                Redo
            </button>
            <select
                onChange={(e) =>
                    onCheckpointLoad(
                        history.resetToCheckpoint(Number(e.target.value))
                    )
                }
                value={history.currentCheckpoint.id}
            >
                {history.checkpoints.map((checkpoint) => {
                    return (
                        <option key={checkpoint.id} value={checkpoint.id}>
                            {checkpoint.description}
                        </option>
                    );
                })}
            </select>
        </div>
    );
}

export default HistoryNavigator;
