import React from "react";
import { ReactNode } from "react";
import { NodeType } from "../../common/graph/nodetype";
import "./nodetypeentry.css";
import { DynamicGraph } from "../graph";

type propTypes = {
    graph: DynamicGraph;
    type: NodeType;
    onChange: { (): void };
    onSelectAll: (type: NodeType) => void;
};
type stateTypes = {
    temporaryColor: string;
};

export class NodeTypeEntry extends React.Component<propTypes, stateTypes> {
    debounceTimeout: NodeJS.Timeout;
    debounceArgs: any[];
    debounceFunc: any;

    constructor(props: propTypes) {
        super(props);
        this.deleteType = this.deleteType.bind(this);
        this.handleTextChange = this.handleTextChange.bind(this);
        this.handleColorChange = this.handleColorChange.bind(this);

        this.state = {
            temporaryColor: undefined,
        };
    }

    private deleteType() {
        this.props.type.delete();
    }

    private isValidColor(color: string): boolean {
        if (color.length <= 0) {
            return false;
        }

        // Source: https://stackoverflow.com/a/8027444
        return /^#([0-9A-F]{3}){1,2}$/i.test(color);
    }

    private handleColorChange(event: any) {
        const newColor = event.target.value;

        if (this.isValidColor(newColor)) {
            // Is actual change?
            if (this.props.type.color !== newColor) {
                // Update proper color
                this.props.type.color = newColor;
                // this.props.type.graph.storeCurrentData( // TODO: Reimplement
                //     "Changed color of type [" + this.props.type + "]"
                // );
            }

            // Reset temporary value
            this.setState({
                temporaryColor: undefined,
            });
        } else {
            // Only set as temporary value
            this.setState({
                temporaryColor: newColor,
            });
        }

        this.props.onChange();
    }

    /**
     * Generic function for handeling a changing text input and applying the new value to the node type.
     * @param event Change event of text input.
     * @param property Property to give new value.
     */
    private handleTextChange(event: any, property: string) {
        const newValue = event.target.value;

        // Actual change?
        if ((this.props.type as any)[property] == newValue) {
            return;
        }

        (this.props.type as any)[property] = newValue;
        this.props.onChange();

        // Save change, but debounce, so it doesn't trigger too quickly
        this.props.onChange();
        // this.debounce(
        //     (property: string) => {
        //         // this.props.type.graph.storeCurrentData( // TODO: Reimplement
        //         //     "Changed " + property + " of type [" + this.props.type + "]"
        //         // );
        //         this.props.onChange();
        //     },
        //     500,
        //     property
        // );
    }

    // debounce(func: any, wait: number, ...args: any[]) {
    //     // It works, don't question it
    //     const later = () => {
    //         this.debounceTimeout = null;
    //         this.debounceFunc(...this.debounceArgs);
    //     };
    //
    //     clearTimeout(this.debounceTimeout);
    //     if (
    //         this.debounceArgs !== undefined &&
    //         args[0] !== this.debounceArgs[0] &&
    //         this.debounceFunc !== undefined
    //     ) {
    //         this.debounceFunc(...this.debounceArgs);
    //     }
    //
    //     this.debounceArgs = args;
    //     this.debounceFunc = func;
    //     this.debounceTimeout = setTimeout(later, wait);
    // }

    render(): ReactNode {
        return (
            <li className="node-type">
                <input
                    className="node-type-name"
                    type={"text"}
                    value={this.props.type.name}
                    onChange={(event) => this.handleTextChange(event, "name")}
                />
                <input
                    className="node-type-color"
                    type={"text"}
                    value={
                        this.state.temporaryColor !== undefined
                            ? this.state.temporaryColor
                            : this.props.type.color
                    }
                    onChange={(event) => this.handleColorChange(event)}
                />
                <button onClick={() => this.props.onSelectAll(this.props.type)}>
                    Select nodes
                </button>
                {this.props.graph &&
                this.props.graph.objectGroups.length > 1 ? (
                    <button onClick={this.deleteType}>Delete</button>
                ) : (
                    ""
                )}
            </li>
        );
    }
}
